@@ -20,19 +20,19 @@ from scipy.stats import chi2, f
 from samplics.estimation import TaylorEstimator
 from samplics.utils.basic_functions import set_variables_names
 from samplics.utils.formats import concatenate_series_to_str, numpy_array, remove_nans
-from samplics.utils.types import Array, Number, Series, StringNumber
+from samplics.utils.types import Array, Number, Series, StringNumber, SinglePSUEst
 
 
 class Tabulation:
     def __init__(
         self,
-        parameter: str = "count",
+        param: str = "count",
         alpha: float = 0.05,
         ciprop_method: str = "logit",
     ) -> None:
 
-        if parameter.lower() in ("count", "proportion"):
-            self.parameter = parameter.lower()
+        if param.lower() in ("count", "proportion"):
+            self.param = param.lower()
         else:
             raise ValueError("parameter must be 'count' or 'proportion'")
         self.type = "oneway"
@@ -49,7 +49,7 @@ class Tabulation:
         self.vars_levels: dict[str, list[StringNumber]] = {}
 
     def __repr__(self) -> str:
-        return f"Tabulation(parameter={self.parameter}, alpha={self.alpha})"
+        return f"Tabulation(parameter={self.param}, alpha={self.alpha})"
 
     def __str__(self) -> str:
 
@@ -57,9 +57,9 @@ class Tabulation:
             return "No categorical variables to tabulate"
         else:
             tbl_head = f"Tabulation of {self.vars_names[0]}"
-            tbl_subhead1 = f" Number of strata: {self.design_info['number_strata']}"
-            tbl_subhead2 = f" Number of PSUs: {self.design_info['number_psus']}"
-            tbl_subhead3 = f" Number of observations: {self.design_info['number_obs']}"
+            tbl_subhead1 = f" Number of strata: {self.design_info['nb_strata']}"
+            tbl_subhead2 = f" Number of PSUs: {self.design_info['nb_psus']}"
+            tbl_subhead3 = f" Number of observations: {self.design_info['nb_obs']}"
             tbl_subhead4 = f" Degrees of freedom: {self.design_info['degrees_of_freedom']:.2f}"
 
             return f"\n{tbl_head}\n{tbl_subhead1}\n{tbl_subhead2}\n{tbl_subhead3}\n{tbl_subhead4}\n\n {self.to_dataframe().to_string(index=False)}\n"
@@ -74,7 +74,9 @@ class Tabulation:
         ssu: Optional[Array] = None,
         fpc: Union[dict, float] = 1,
         deff: bool = False,
-        coef_variation: bool = False,
+        coef_var: bool = False,
+        single_psu: Union[SinglePSUEst, dict[StringNumber, SinglePSUEst]] = SinglePSUEst.error,
+        strata_comb: Optional[dict[Array, Array]] = None,
         remove_nan: bool = False,
     ) -> tuple[TaylorEstimator, list, int]:
 
@@ -90,8 +92,8 @@ class Tabulation:
 
         var_of_ones = numpy_array(var_of_ones)
 
-        if self.parameter == "count":
-            tbl_est = TaylorEstimator(parameter="total", alpha=self.alpha)
+        if self.param == "count":
+            tbl_est = TaylorEstimator(param="total", alpha=self.alpha)
             tbl_est.estimate(
                 y=var_of_ones,
                 samp_weight=samp_weight,
@@ -101,11 +103,13 @@ class Tabulation:
                 domain=var,
                 fpc=fpc,
                 deff=deff,
-                coef_variation=coef_variation,
+                coef_var=coef_var,
+                single_psu=single_psu,
+                strata_comb=strata_comb,
                 remove_nan=False,
             )
-        elif self.parameter == "proportion":
-            tbl_est = TaylorEstimator(parameter=self.parameter, alpha=self.alpha)
+        elif self.param == "proportion":
+            tbl_est = TaylorEstimator(param=self.param, alpha=self.alpha)
             tbl_est.estimate(
                 y=var,
                 samp_weight=samp_weight,
@@ -114,7 +118,9 @@ class Tabulation:
                 ssu=ssu,
                 fpc=fpc,
                 deff=deff,
-                coef_variation=coef_variation,
+                coef_var=coef_var,
+                single_psu=single_psu,
+                strata_comb=strata_comb,
                 remove_nan=False,
             )
         else:
@@ -130,10 +136,12 @@ class Tabulation:
         stratum: Optional[Array] = None,
         psu: Optional[Array] = None,
         ssu: Optional[Array] = None,
-        # Todo: by: Optional[Array] = None,
+        # TODO: by: Optional[Array] = None,
         fpc: Union[dict, float] = 1,
         deff: bool = False,
-        coef_variation: bool = False,
+        coef_var: bool = False,
+        single_psu: Union[SinglePSUEst, dict[StringNumber, SinglePSUEst]] = SinglePSUEst.error,
+        strata_comb: Optional[dict[Array, Array]] = None,
         remove_nan: bool = False,
     ) -> None:
         """
@@ -179,17 +187,19 @@ class Tabulation:
                 ssu=ssu,
                 fpc=fpc,
                 deff=deff,
-                coef_variation=coef_variation,
+                coef_var=coef_var,
+                single_psu=single_psu,
+                strata_comb=strata_comb,
                 remove_nan=remove_nan,
             )
             self.vars_levels[vars_names[0]] = var_levels
-            if self.parameter == "count":
+            if self.param == "count":
                 self.point_est[vars_names[0]] = tbl_est.point_est
                 self.stderror[vars_names[0]] = tbl_est.stderror
                 self.lower_ci[vars_names[0]] = tbl_est.lower_ci
                 self.upper_ci[vars_names[0]] = tbl_est.upper_ci
                 self.deff[vars_names[0]] = {}  # todo: tbl_est.deff
-            elif self.parameter == "proportion":
+            elif self.param == "proportion":
                 self.point_est[vars_names[0]] = tbl_est.point_est
                 self.stderror[vars_names[0]] = tbl_est.stderror
                 self.lower_ci[vars_names[0]] = tbl_est.lower_ci
@@ -208,17 +218,19 @@ class Tabulation:
                     ssu=ssu,
                     fpc=fpc,
                     deff=deff,
-                    coef_variation=coef_variation,
+                    coef_var=coef_var,
+                    single_psu=single_psu,
+                    strata_comb=strata_comb,
                     remove_nan=remove_nan,
                 )
                 self.vars_levels[vars_names[k]] = var_levels
-                if self.parameter == "count":
+                if self.param == "count":
                     self.point_est[vars_names[k]] = tbl_est.point_est
                     self.stderror[vars_names[k]] = tbl_est.stderror
                     self.lower_ci[vars_names[k]] = tbl_est.lower_ci
                     self.upper_ci[vars_names[k]] = tbl_est.upper_ci
                     self.deff[vars_names[k]] = {}  # todo: tbl_est.deff
-                elif self.parameter == "proportion":
+                elif self.param == "proportion":
                     self.point_est[vars_names[k]] = tbl_est.point_est
                     self.stderror[vars_names[k]] = tbl_est.stderror
                     self.lower_ci[vars_names[k]] = tbl_est.lower_ci
@@ -227,11 +239,11 @@ class Tabulation:
 
         self.vars_names = vars_names
         self.design_info = {
-            "number_strata": tbl_est.number_strata,
-            "number_psus": tbl_est.number_psus,
-            "number_obs": nb_obs,
+            "nb_strata": tbl_est.nb_strata,
+            "nb_psus": tbl_est.nb_psus,
+            "nb_obs": nb_obs,
             "design_effect": 0,
-            "degrees_of_freedom": tbl_est.number_psus - tbl_est.number_strata,
+            "degrees_of_freedom": tbl_est.nb_psus - tbl_est.nb_strata,
         }
 
     def to_dataframe(
@@ -243,7 +255,7 @@ class Tabulation:
         for var in self.vars_names:
             var_df = pd.DataFrame(np.repeat(var, len(self.vars_levels[var])), columns=["variable"])
             var_df["category"] = self.vars_levels[var]
-            var_df[self.parameter] = list(self.point_est[var].values())
+            var_df[self.param] = list(self.point_est[var].values())
             var_df["stderror"] = list(self.stderror[var].values())
             var_df["lower_ci"] = list(self.lower_ci[var].values())
             var_df["upper_ci"] = list(self.upper_ci[var].values())
@@ -270,13 +282,13 @@ class CrossTabulation:
 
     def __init__(
         self,
-        parameter: str = "count",
+        param: str = "count",
         alpha: float = 0.05,
         ciprop_method: str = "logit",
     ) -> None:
 
-        if parameter.lower() in ("count", "proportion"):
-            self.parameter = parameter.lower()
+        if param.lower() in ("count", "proportion"):
+            self.param = param.lower()
         else:
             raise ValueError("parameter must be 'count' or 'proportion'")
         self.type = "twoway"
@@ -296,7 +308,7 @@ class CrossTabulation:
         self.col_levels: list[StringNumber] = []
 
     def __repr__(self) -> str:
-        return f"CrossTabulation(parameter={self.parameter}, alpha={self.alpha})"
+        return f"CrossTabulation(parameter={self.param}, alpha={self.alpha})"
 
     def __str__(self) -> str:
 
@@ -304,9 +316,9 @@ class CrossTabulation:
             return "No categorical variables to tabulate"
         else:
             tbl_head = f"Cross-tabulation of {self.vars_names[0]} and {self.vars_names[1]}"
-            tbl_subhead1 = f" Number of strata: {self.design_info['number_strata']}"
-            tbl_subhead2 = f" Number of PSUs: {self.design_info['number_psus']}"
-            tbl_subhead3 = f" Number of observations: {self.design_info['number_obs']}"
+            tbl_subhead1 = f" Number of strata: {self.design_info['nb_strata']}"
+            tbl_subhead2 = f" Number of PSUs: {self.design_info['nb_psus']}"
+            tbl_subhead3 = f" Number of observations: {self.design_info['nb_obs']}"
             tbl_subhead4 = f" Degrees of freedom: {self.design_info['degrees_of_freedom']:.2f}"
 
             chisq_dist = f"chi2({self.stats['Pearson-Unadj']['df']})"
@@ -324,6 +336,34 @@ class CrossTabulation:
 
             return f"\n{tbl_head}\n{tbl_subhead1}\n{tbl_subhead2}\n{tbl_subhead3}\n{tbl_subhead4}\n\n {self.to_dataframe().to_string(index=False)}\n\n{pearson_test}\n\n {lr_test}\n"
 
+    # also mutates tbl_est
+    def _extract_estimates(self, tbl_est, vars_levels) -> tuple[np.ndarray, np.ndarray]:
+
+        levels = list(tbl_est.point_est.keys())
+        missing_levels = vars_levels[~np.isin(vars_levels, levels)]
+        if missing_levels.shape[0] > 0:
+            for level in vars_levels:
+                if level in missing_levels:
+                    tbl_est.point_est[level] = 0.0
+                    tbl_est.stderror[level] = 0.0
+                    tbl_est.lower_ci[level] = 0.0
+                    tbl_est.upper_ci[level] = 0.0
+                    tbl_est.covariance[level] = {}
+                    for ll in vars_levels:
+                        tbl_est.covariance[level][ll] = 0.0
+                else:
+                    for ll in missing_levels:
+                        tbl_est.covariance[level][ll] = 0.0
+                    tbl_est.covariance[level] = dict(sorted(tbl_est.covariance[level].items()))
+
+        _tbl_est_point_est = dict(sorted(tbl_est.point_est.items()))
+        _tbl_est_covariance = dict(sorted(tbl_est.covariance.items()))
+        return (
+            np.array(list(_tbl_est_point_est.values())),
+            pd.DataFrame.from_dict(_tbl_est_covariance, orient="index").to_numpy(),
+            missing_levels,
+        )
+
     def tabulate(
         self,
         vars: Array,
@@ -335,7 +375,9 @@ class CrossTabulation:
         # Todo: by: Optional[Array] = None,
         fpc: Union[dict, float] = 1,
         deff: bool = False,
-        coef_variation: bool = False,
+        coef_var: bool = False,
+        single_psu: Union[SinglePSUEst, dict[StringNumber, SinglePSUEst]] = SinglePSUEst.error,
+        strata_comb: Optional[dict[Array, Array]] = None,
         remove_nan: bool = False,
     ) -> None:
 
@@ -402,6 +444,9 @@ class CrossTabulation:
 
         vars_dummies = np.asarray(dmatrix(two_way_full_model, vars_levels, NA_action="raise"))
 
+        # vars_dummies = np.delete(vars_dummies, obj=1, axis=0)
+        # vars_dummies = np.delete(vars_dummies, obj=2, axis=1)
+
         if len(vars.shape) == 2:
             vars_for_oneway = np.apply_along_axis(
                 func1d=concatenate_series_to_str, axis=1, arr=vars
@@ -413,7 +458,7 @@ class CrossTabulation:
             func1d=concatenate_series_to_str, axis=1, arr=vars_levels
         )
 
-        tbl_est_prop = TaylorEstimator(parameter="mean", alpha=self.alpha)
+        tbl_est_prop = TaylorEstimator(param="mean", alpha=self.alpha)
         tbl_est_prop.estimate(
             y=vars_for_oneway,
             samp_weight=samp_weight,
@@ -421,19 +466,28 @@ class CrossTabulation:
             psu=psu,
             ssu=ssu,
             fpc=fpc,
+            coef_var=coef_var,
+            single_psu=single_psu,
+            strata_comb=strata_comb,
             as_factor=True,
         )
+
         tbl_est = tbl_est_prop
-        cell_est = np.array(list(tbl_est_prop.point_est.values()))
+        # self._extract_estimates() also mutates tbl_est
+        cell_est, cov_prop, missing_levels = self._extract_estimates(
+            tbl_est=tbl_est_prop, vars_levels=vars_levels_concat
+        )
         cov_prop_srs = (
             np.diag(cell_est)
-            - cell_est.reshape(vars_levels.shape[0], 1)
-            @ np.transpose(cell_est.reshape(vars_levels.shape[0], 1))
+            # - cell_est.reshape(vars_levels.shape[0], 1)
+            # - cell_est.reshape(cell_est.shape[0], 1)
+            # @ np.transpose(cell_est.reshape(vars_levels.shape[0], 1))
+            # @ cell_est.reshape(1, cell_est.shape[0])
         ) / vars.shape[0]
-        cov_prop = pd.DataFrame.from_dict(tbl_est_prop.covariance, orient="index").to_numpy()
+        # breakpoint()
 
-        if self.parameter == "count":
-            tbl_est_count = TaylorEstimator(parameter="total", alpha=self.alpha)
+        if self.param == "count":
+            tbl_est_count = TaylorEstimator(param="total", alpha=self.alpha)
             tbl_est_count.estimate(
                 y=vars_for_oneway,
                 samp_weight=samp_weight,
@@ -441,24 +495,30 @@ class CrossTabulation:
                 psu=psu,
                 ssu=ssu,
                 fpc=fpc,
+                coef_var=coef_var,
+                single_psu=single_psu,
+                strata_comb=strata_comb,
                 as_factor=True,
             )
             tbl_est = tbl_est_count
-
+        # breakpoint()
         nrows = row_levels.__len__()
         ncols = col_levels.__len__()
-        x1 = vars_dummies[:, 1 : (nrows - 1) + (ncols - 1) + 1]  # main_effects
+        x1 = vars_dummies[:, 0 : (nrows - 1) + (ncols - 1) + 1]  # main_effects
         x2 = vars_dummies[:, (nrows - 1) + (ncols - 1) + 1 :]  # interactions
         x1_t = np.transpose(x1)
+        # breakpoint()
         x2_tilde = x2 - x1 @ np.linalg.inv(x1_t @ cov_prop_srs @ x1) @ (x1_t @ cov_prop_srs @ x2)
         delta_est = np.linalg.inv(np.transpose(x2_tilde) @ cov_prop_srs @ x2_tilde) @ (
             np.transpose(x2_tilde) @ cov_prop @ x2_tilde
         )
+        # breakpoint()
         tbl_keys = list(tbl_est.point_est.keys())
         cell_est = np.zeros(vars_levels.shape[0])
         cell_stderror = np.zeros(vars_levels.shape[0])
         cell_lower_ci = np.zeros(vars_levels.shape[0])
         cell_upper_ci = np.zeros(vars_levels.shape[0])
+        # breakpoint()
         for k in range(vars_levels.shape[0]):
             if vars_levels_concat[k] in tbl_keys:
                 cell_est[k] = tbl_est.point_est[vars_levels_concat[k]]
@@ -499,24 +559,30 @@ class CrossTabulation:
 
         point_est_df = pd.DataFrame.from_dict(self.point_est, orient="index").values
 
-        if self.parameter == "count":
+        if self.param == "count":
             point_est_df = point_est_df / np.sum(point_est_df)
-        point_est_null = point_est_df.sum(axis=1).reshape(nrows, 1) @ np.transpose(
-            point_est_df.sum(axis=0).reshape(ncols, 1)
-        )
+
+        point_est_null = point_est_df.sum(axis=1).reshape(nrows, 1) @ point_est_df.sum(
+            axis=0
+        ).reshape(1, ncols)
 
         chisq_p = float(
             vars.shape[0] * np.sum((point_est_df - point_est_null) ** 2 / point_est_null)
         )
         f_p = float(chisq_p / np.trace(delta_est))
+        # breakpoint()
 
-        chisq_lr = float(
-            2 * vars.shape[0] * np.sum(point_est_df * np.log(point_est_df / point_est_null))
-        )
+        # valid indexes (i,j) correspond to n_ij > 0
+        valid_indx = (point_est_df != 0) & (point_est_null != 0)
+
+        log_mat = np.zeros(point_est_null.shape)
+        log_mat[valid_indx] = np.log(point_est_df[valid_indx] / point_est_null[valid_indx])
+
+        chisq_lr = float(2 * vars.shape[0] * np.sum(point_est_df * log_mat))
         f_lr = float(chisq_lr / np.trace(delta_est))
 
         df_num = float((np.trace(delta_est) ** 2) / np.trace(delta_est @ delta_est))
-        df_den = float((tbl_est.number_psus - tbl_est.number_strata) * df_num)
+        df_den = float((tbl_est.nb_psus - tbl_est.nb_strata) * df_num)
 
         self.stats = {
             "Pearson-Unadj": {
@@ -547,13 +613,15 @@ class CrossTabulation:
         self.col_levels = list(col_levels)
         self.vars_names = vars_names
         self.design_info = {
-            "number_strata": tbl_est.number_strata,
-            "number_psus": tbl_est.number_psus,
-            "number_obs": vars.shape[0],
+            "nb_strata": tbl_est.nb_strata,
+            "nb_psus": tbl_est.nb_psus,
+            "nb_obs": vars.shape[0],
             "design_effect": 0,
-            "degrees_of_freedom": tbl_est.number_psus - tbl_est.number_strata,
+            "degrees_of_freedom": tbl_est.nb_psus - tbl_est.nb_strata,
         }
 
+        # breakpoint()
+
     def to_dataframe(
         self,
     ) -> pd.DataFrame:
@@ -564,7 +632,7 @@ class CrossTabulation:
 
         for _ in range(len(self.row_levels)):
             for _ in range(len(self.col_levels)):
-                twoway_df[self.parameter] = sum(
+                twoway_df[self.param] = sum(
                     pd.DataFrame.from_dict(self.point_est, orient="index").values.tolist(), []
                 )
                 twoway_df["stderror"] = sum(
