@@ -22,25 +22,26 @@ import pandas as pd
 
 from scipy.stats import t as student
 
+from samplics.utils.basic_functions import get_single_psu_strata
 from samplics.utils.formats import dict_to_dataframe, fpc_as_dict, numpy_array, remove_nans
-from samplics.utils.types import Array, Number, Series, StringNumber
+from samplics.utils.types import Array, Number, Series, StringNumber, SinglePSUEst, SamplicsError
 
 
 class _SurveyEstimator:
     """General approach for sample estimation of linear parameters."""
 
-    def __init__(self, parameter: str, alpha: float = 0.05, random_seed: Optional[int] = None):
-        """Initializes the instance """
+    def __init__(self, param: str, alpha: float = 0.05, rand_seed: Optional[int] = None):
+        """Initializes the instance"""
 
-        self.random_seed: Optional[int]
-        if random_seed is not None:
-            self.random_seed = random_seed
-            np.random.seed(self.random_seed)
+        self.rand_seed: Optional[int]
+        if rand_seed is not None:
+            self.rand_seed = rand_seed
+            np.random.seed(self.rand_seed)
         else:
-            self.random_seed = None
+            self.rand_seed = None
 
-        if parameter.lower() in ("proportion", "mean", "total", "ratio"):
-            self.parameter = parameter.lower()
+        if param.lower() in ("proportion", "mean", "total", "ratio"):
+            self.param = param.lower()
         else:
             raise AssertionError("parameter must be 'proportion', 'mean', 'total' or 'ratio'")
         self.alpha = alpha
@@ -54,20 +55,21 @@ class _SurveyEstimator:
         self.lower_ci: Any = {}
         self.upper_ci: Any = {}
         self.fpc: Any = {}
-        self.strata: Any = []
+        self.strata: Any = None
+        self.single_psu_strata: Any = None
         self.domains: Any = None
         self.method: Any = "taylor"
-        self.number_strata: Any = None
-        self.number_psus: Any = None
+        self.nb_strata: Any = None
+        self.nb_psus: Any = None
         self.degree_of_freedom: Any = None
         self.as_factor: bool = False  # whether to treat outcome as factor.
 
     def __str__(self) -> Any:
-        print(f"SAMPLICS - Estimation of {self.parameter.title()}\n")
-        print(f"Number of strata: {self.number_strata}")
-        print(f"Number of psus: {self.number_psus}")
+        print(f"SAMPLICS - Estimation of {self.param.title()}\n")
+        print(f"Number of strata: {self.nb_strata}")
+        print(f"Number of psus: {self.nb_psus}")
         print(f"Degree of freedom: {self.degree_of_freedom}\n")
-        parameter = self.parameter.upper()
+        param = self.param.upper()
         estimation = pd.DataFrame()
         if (
             isinstance(self.point_est, dict)
@@ -76,7 +78,7 @@ class _SurveyEstimator:
             and isinstance(self.upper_ci, dict)
             and isinstance(self.coef_var, dict)
         ):
-            if self.domains is not None and (self.parameter == "proportion" or self.as_factor):
+            if self.domains is not None and (self.param == "proportion" or self.as_factor):
                 domains = list()
                 levels = list()
                 point_est = list()
@@ -94,27 +96,27 @@ class _SurveyEstimator:
                     coef_var += list(self.coef_var[domain].values())
                 estimation["DOMAIN"] = domains
                 estimation["LEVEL"] = levels
-                estimation[parameter] = point_est
+                estimation[param] = point_est
                 estimation["SE"] = stderror
                 estimation["LCI"] = lower_ci
                 estimation["UCI"] = upper_ci
                 estimation["CV"] = coef_var
             elif self.domains is not None:
                 estimation["DOMAIN"] = self.domains
-                estimation[parameter] = self.point_est.values()
+                estimation[param] = self.point_est.values()
                 estimation["SE"] = self.stderror.values()
                 estimation["LCI"] = self.lower_ci.values()
                 estimation["UCI"] = self.upper_ci.values()
                 estimation["CV"] = self.coef_var.values()
             else:
                 estimation["LEVELS"] = self.point_est.keys()
-                estimation[parameter] = self.point_est.values()
+                estimation[param] = self.point_est.values()
                 estimation["SE"] = self.stderror.values()
                 estimation["LCI"] = self.lower_ci.values()
                 estimation["UCI"] = self.upper_ci.values()
                 estimation["CV"] = self.coef_var.values()
         else:
-            estimation[parameter] = [self.point_est]
+            estimation[param] = [self.point_est]
             estimation["SE"] = [self.stderror]
             estimation["LCI"] = [self.lower_ci]
             estimation["UCI"] = [self.upper_ci]
@@ -136,32 +138,32 @@ class _SurveyEstimator:
         psu = numpy_array(psu)
 
         if stratum.size <= 1:
-            self.number_psus = np.unique(psu).size if psu.size > 1 else samp_weight.size
-            self.number_strata = 1
+            self.nb_psus = np.unique(psu).size if psu.size > 1 else samp_weight.size
+            self.nb_strata = 1
         elif psu.size > 1:
             # if type object ("O") change it to str, otherwise np.unique will fail
             if stratum.dtype == "O":
                 stratum = stratum.astype(str)
             if psu.dtype == "O":
                 psu = psu.astype(str)
-            self.number_psus = np.unique([stratum, psu], axis=1).shape[1]
-            self.number_strata = np.unique(stratum).size
+            self.nb_psus = np.unique([stratum, psu], axis=1).shape[1]
+            self.nb_strata = np.unique(stratum).size
         else:
             samp_weight = numpy_array(samp_weight)
-            self.number_psus = samp_weight.size
-            self.number_strata = np.unique(stratum).size
+            self.nb_psus = samp_weight.size
+            self.nb_strata = np.unique(stratum).size
 
-        self.degree_of_freedom = self.number_psus - self.number_strata
+        self.degree_of_freedom = self.nb_psus - self.nb_strata
 
     def _get_point_d(
         self, y: np.ndarray, samp_weight: np.ndarray, x: Optional[np.ndarray] = None
     ) -> float:
 
-        if self.parameter in ("proportion", "mean"):
+        if self.param in ("proportion", "mean"):
             return float(np.sum(samp_weight * y) / np.sum(samp_weight))
-        elif self.parameter == "total":
+        elif self.param == "total":
             return float(np.sum(samp_weight * y))
-        elif self.parameter == "ratio" and x is not None:
+        elif self.param == "ratio" and x is not None:
             return float(np.sum(samp_weight * y) / np.sum(samp_weight * x))
         else:
             raise ValueError("Parameter not valid!")
@@ -190,17 +192,17 @@ class _SurveyEstimator:
 
         """
 
-        if self.parameter == "ratio" and x is None:
+        if self.param == "ratio" and x is None:
             raise AssertionError("Parameter x must be provided for ratio estimation.")
 
         if remove_nan:
-            if self.parameter == "ratio" and x is not None:
+            if self.param == "ratio" and x is not None:
                 excluded_units = np.isnan(y) | np.isnan(x)
             else:
                 excluded_units = np.isnan(y)
             y, samp_weight, x, domain = remove_nans(excluded_units, y, samp_weight, x, domain)
 
-        if self.parameter == "proportion" or as_factor:
+        if self.param == "proportion" or as_factor:
             y_dummies = pd.get_dummies(y)
             categories = y_dummies.columns
             y_dummies = y_dummies.values
@@ -209,7 +211,7 @@ class _SurveyEstimator:
             y_dummies = None
 
         if domain is None:
-            if self.parameter == "proportion" or as_factor:
+            if self.param == "proportion" or as_factor:
                 cat_dict: dict[StringNumber, float] = {}
                 for k in range(categories.size):
                     y_k = y_dummies[:, k]
@@ -220,7 +222,7 @@ class _SurveyEstimator:
                 return self._get_point_d(y, samp_weight, x)
         else:
             domain_ids = np.unique(domain)
-            if self.parameter == "proportion" or as_factor:
+            if self.param == "proportion" or as_factor:
                 estimate1: dict[StringNumber, dict[StringNumber, float]] = {}
                 for d in domain_ids:
                     weight_d = samp_weight[domain == d]
@@ -237,7 +239,7 @@ class _SurveyEstimator:
                     weight_d = samp_weight[domain == d]
                     y_d = y[domain == d]
                     if x is not None:
-                        x_d = x[domain == d] if self.parameter == "ratio" else None
+                        x_d = x[domain == d] if self.param == "ratio" else None
                     else:
                         x_d = None
                     estimate2[d] = self._get_point_d(y_d, weight_d, x_d)
@@ -260,9 +262,9 @@ class TaylorEstimator(_SurveyEstimator):
         | strata (list): list of the strata in the sample.
         | domains (list): list of the domains in the sample.
         | method (str): variance estimation method.
-        | parameter (str): the parameter of the population to estimate e.g. total.
-        | number_strata (int): number of strata.
-        | number_psus (int): number of primary sampling units (psus)
+        | param (str): the parameter of the population to estimate e.g. total.
+        | nb_strata (int): number of strata.
+        | nb_psus (int): number of primary sampling units (psus)
 
     Methods
         | estimate(): produces the point estimate of the parameter of interest with the associated
@@ -271,14 +273,14 @@ class TaylorEstimator(_SurveyEstimator):
 
     def __init__(
         self,
-        parameter: str,
+        param: str,
         alpha: float = 0.05,
-        random_seed: Optional[int] = None,
+        rand_seed: Optional[int] = None,
         ciprop_method: Optional[str] = "logit",
     ) -> None:
-        """Initializes the instance """
-        _SurveyEstimator.__init__(self, parameter)
-        if self.parameter == "proportion":
+        """Initializes the instance"""
+        _SurveyEstimator.__init__(self, param)
+        if self.param == "proportion":
             self.ciprop_method = ciprop_method
         else:
             self.ciprop_method = None
@@ -295,17 +297,17 @@ class TaylorEstimator(_SurveyEstimator):
         ncols = 1 if len(y.shape) == 1 else y.shape[1]
         y = y.reshape(y.shape[0], ncols)
         y_weighted = y * samp_weight[:, None]  # .reshape(samp_weight.shape[0], 1)
-        if self.parameter in ("proportion", "mean"):
+        if self.param in ("proportion", "mean"):
             scale_weights = np.sum(samp_weight)
             location_weights = np.sum(y_weighted, axis=0) / scale_weights
             return np.asarray((y - location_weights) * samp_weight[:, None] / scale_weights)
-        elif self.parameter == "ratio":
+        elif self.param == "ratio":
             weighted_sum_x = np.sum(x * samp_weight)
             weighted_ratio = np.sum(y_weighted, axis=0) / weighted_sum_x
             return np.asarray(
                 samp_weight[:, None] * (y - x[:, None] * weighted_ratio) / weighted_sum_x
             )
-        elif self.parameter == "total":
+        elif self.param == "total":
             return np.asarray(y_weighted)
         else:
             raise ValueError("parameter not valid!")
@@ -314,38 +316,39 @@ class TaylorEstimator(_SurveyEstimator):
     def _variance_stratum_between(
         y_score_s: np.ndarray,
         samp_weight_s: np.ndarray,
-        number_psus_in_s: int,
+        nb_psus_in_s: int,
         psu_s: Optional[np.ndarray],
     ) -> np.ndarray:
-        """Computes the variance for one stratum """
+        """Computes the variance for one stratum"""
 
         covariance = np.asarray([])
-        if number_psus_in_s > 1:
-            scores_s_mean = np.asarray(y_score_s.sum(axis=0) / number_psus_in_s)  # new
+        if psu_s is not None:
+            scores_s_mean = np.asarray(y_score_s.sum(axis=0) / nb_psus_in_s)  # new
             psus = np.unique(psu_s)
-            scores_psus_sums = np.zeros((number_psus_in_s, scores_s_mean.shape[0]))
+            scores_psus_sums = np.zeros((nb_psus_in_s, scores_s_mean.shape[0]))
             for k, psu in enumerate(np.unique(psus)):
                 scores_psus_sums[k, :] = y_score_s[psu_s == psu].sum(axis=0)
             covariance = np.transpose(scores_psus_sums - scores_s_mean) @ (
                 scores_psus_sums - scores_s_mean
             )
-            covariance = (number_psus_in_s / (number_psus_in_s - 1)) * covariance
-        elif number_psus_in_s in (0, 1):
-            number_obs = y_score_s.shape[0]
-            y_score_s_mean = y_score_s.sum(axis=0) / number_obs
+            covariance = (nb_psus_in_s / (nb_psus_in_s - 1)) * covariance
+        else:
+            nb_obs = y_score_s.shape[0]
+            y_score_s_mean = y_score_s.sum(axis=0) / nb_obs
             covariance = (
-                (number_obs / (number_obs - 1))
+                (nb_obs / (nb_obs - 1))
                 * np.transpose(y_score_s - y_score_s_mean)
                 @ (y_score_s - y_score_s_mean)
             )
-        else:
-            raise ValueError("Number of psus cannot be negative.")
 
         return covariance
 
     @staticmethod
     def _variance_stratum_within(
-        y_score_s: np.ndarray, number_psus_in_s: np.ndarray, psu_s: np.ndarray, ssu_s: np.ndarray,
+        y_score_s: np.ndarray,
+        nb_psus_in_s: np.ndarray,
+        psu_s: np.ndarray,
+        ssu_s: np.ndarray,
     ) -> float:
 
         variance = 0.0
@@ -355,12 +358,12 @@ class TaylorEstimator(_SurveyEstimator):
             for psu in np.unique(psus):
                 scores_psu_mean = y_score_s[psus == psu].mean()
                 ssus = np.unique(ssu_s[psu_s == psu])
-                number_ssus_in_psu = np.size(ssus)
-                scores_ssus_sums = np.zeros(number_ssus_in_psu)
-                if number_ssus_in_psu > 1:
+                nb_ssus_in_psu = np.size(ssus)
+                scores_ssus_sums = np.zeros(nb_ssus_in_psu)
+                if nb_ssus_in_psu > 1:
                     for k, ssu in enumerate(np.unique(ssus)):
                         scores_ssus_sums[k] = y_score_s[ssu_s == ssu].sum()
-                    variance += (number_ssus_in_psu / (number_ssus_in_psu - 1)) * (
+                    variance += (nb_ssus_in_psu / (nb_ssus_in_psu - 1)) * (
                         (scores_ssus_sums - scores_psu_mean) ** 2
                     ).sum()
 
@@ -374,24 +377,36 @@ class TaylorEstimator(_SurveyEstimator):
         psu: Optional[np.ndarray],
         ssu: Optional[np.ndarray] = None,
         fpc: Union[dict[StringNumber, Number], Number] = 1,
+        skipped_strata: Optional[np.ndarray] = None,
     ) -> np.ndarray:
-        """Computes the variance across stratum """
+        """Computes the variance across stratum"""
 
         if stratum is None and isinstance(fpc, (int, float)):
-            number_psus = np.unique(psu).size
+            nb_psus = np.unique(psu).size
             return np.asarray(
-                fpc * self._variance_stratum_between(y_score, samp_weight, number_psus, psu)
+                fpc
+                * self._variance_stratum_between(
+                    y_score_s=y_score,
+                    samp_weight_s=samp_weight,
+                    nb_psus_in_s=nb_psus,
+                    psu_s=psu,
+                )
             )
         elif isinstance(fpc, dict):
             covariance = np.zeros((y_score.shape[1], y_score.shape[1]))
-            for s in np.unique(stratum):
+            strata = np.unique(stratum)
+            singletons = np.isin(strata, skipped_strata)
+            for s in strata[~singletons]:
                 y_score_s = y_score[stratum == s]
                 samp_weight_s = samp_weight[stratum == s]
                 psu_s = psu[stratum == s] if psu is not None else None  # np.array([])
-                number_psus_in_s = np.size(np.unique(psu_s))
+                nb_psus_in_s = np.size(np.unique(psu_s)) if psu_s is not None else 0
                 ssu_s = ssu[stratum == s] if ssu is not None else None
                 covariance += fpc[s] * self._variance_stratum_between(
-                    y_score_s, samp_weight_s, number_psus_in_s, psu_s
+                    y_score_s=y_score_s,
+                    samp_weight_s=samp_weight_s,
+                    nb_psus_in_s=nb_psus_in_s,
+                    psu_s=psu_s,
                 )
             return np.asarray(covariance)
         else:
@@ -407,17 +422,19 @@ class TaylorEstimator(_SurveyEstimator):
         ssu: Optional[np.ndarray] = None,
         domain: Optional[np.ndarray] = None,
         fpc: Union[dict[StringNumber, Number], Number] = 1,
+        skipped_strata: Optional[np.ndarray] = None,
         as_factor: bool = False,
         remove_nan: bool = False,
     ) -> tuple[
-        Union[dict[StringNumber, Any], np.ndarray], Union[dict[StringNumber, Any], np.ndarray],
+        Union[dict[StringNumber, Any], np.ndarray],
+        Union[dict[StringNumber, Any], np.ndarray],
     ]:
 
-        if self.parameter == "ratio" and x is None:
+        if self.param == "ratio" and x is None:
             raise AssertionError("Parameter x must be provided for ratio estimation.")
 
         if remove_nan:
-            if self.parameter == "ratio" and x is not None:
+            if self.param == "ratio" and x is not None:
                 excluded_units = np.isnan(y) | np.isnan(x)
             else:
                 excluded_units = np.isnan(y)
@@ -425,15 +442,23 @@ class TaylorEstimator(_SurveyEstimator):
                     excluded_units, y, samp_weight, x, stratum, domain, psu, ssu
                 )
 
-        if self.parameter == "proportion" or as_factor:
+        if self.param == "proportion" or as_factor:
             y_df = pd.get_dummies(y).astype(int)
             categories = list(y_df.columns)
             y = y_df.values
 
         if domain is None:
             y_score = self._score_variable(y, samp_weight, x)  # new
-            cov_score = self._taylor_variance(y_score, samp_weight, stratum, psu, ssu, fpc)  # new
-            if (self.parameter == "proportion" or as_factor) and isinstance(cov_score, np.ndarray):
+            cov_score = self._taylor_variance(
+                y_score=y_score,
+                samp_weight=samp_weight,
+                stratum=stratum,
+                psu=psu,
+                ssu=ssu,
+                fpc=fpc,
+                skipped_strata=skipped_strata,
+            )  # new
+            if (self.param == "proportion" or as_factor) and isinstance(cov_score, np.ndarray):
                 variance1: dict[StringNumber, dict] = {}
                 covariance1: dict[StringNumber, dict] = {}
                 variance1 = dict(zip(categories, np.diag(cov_score)))
@@ -441,23 +466,30 @@ class TaylorEstimator(_SurveyEstimator):
                     covariance1[level] = dict(zip(categories, cov_score[k, :]))
                 return variance1, covariance1
             else:
-                return cov_score, cov_score  # Todo: generalize for multiple Y variables
+                # Todo: generalize for multiple Y variables
+                return float(cov_score), float(cov_score)
         else:
             domain = np.asarray(domain)
-            if self.parameter == "proportion" or as_factor:
+            if self.param == "proportion" or as_factor:
                 variance2: dict[StringNumber, dict] = {}
                 covariance2: dict[StringNumber, dict] = {}
                 for d in np.unique(domain):
                     domain_d = domain == d
                     weight_d = samp_weight * domain_d
-                    if self.parameter == "ratio":
+                    if self.param == "ratio":
                         x_d = x * domain_d
                     else:
                         x_d = x
                     y_d = y * domain_d if len(y.shape) == 1 else y * domain_d[:, None]
                     y_score_d = self._score_variable(y_d, weight_d, x_d)
                     cov_score_d = self._taylor_variance(
-                        y_score_d, weight_d, stratum, psu, ssu, fpc
+                        y_score=y_score_d,
+                        samp_weight=weight_d,
+                        stratum=stratum,
+                        psu=psu,
+                        ssu=ssu,
+                        fpc=fpc,
+                        skipped_strata=skipped_strata,
                     )
                     variance2[d] = dict(zip(categories, np.diag(cov_score_d)))
                     cov_d = {}
@@ -470,14 +502,20 @@ class TaylorEstimator(_SurveyEstimator):
                 for d in np.unique(domain):
                     domain_d = domain == d
                     weight_d = samp_weight * domain_d
-                    if self.parameter == "ratio":
+                    if self.param == "ratio":
                         x_d = x * domain_d
                     else:
                         x_d = x
                     y_d = y * domain_d if len(y.shape) == 1 else y * domain_d[:, None]
                     y_score_d = self._score_variable(y_d, weight_d, x_d)
                     cov_score_d = self._taylor_variance(
-                        y_score_d, weight_d, stratum, psu, ssu, fpc
+                        y_score=y_score_d,
+                        samp_weight=weight_d,
+                        stratum=stratum,
+                        psu=psu,
+                        ssu=ssu,
+                        fpc=fpc,
+                        skipped_strata=skipped_strata,
                     )
                     variance3[d] = float(cov_score_d)
                 return variance3, variance3
@@ -493,7 +531,8 @@ class TaylorEstimator(_SurveyEstimator):
         domain: Optional[np.ndarray],
         fpc: Union[dict[StringNumber, Number], Number],
         deff: bool,
-        coef_variation: bool,
+        coef_var: bool,
+        skipped_strata: np.ndarray,
         as_factor: bool,
         remove_nan: bool,
     ) -> None:
@@ -515,6 +554,7 @@ class TaylorEstimator(_SurveyEstimator):
             ssu=ssu,
             domain=domain,
             fpc=fpc,
+            skipped_strata=skipped_strata,
             as_factor=as_factor,
             remove_nan=remove_nan,
         )
@@ -524,7 +564,7 @@ class TaylorEstimator(_SurveyEstimator):
 
         if domain is None:
             if (
-                (self.parameter == "proportion" or as_factor and self.parameter == "mean")
+                (self.param == "proportion" or as_factor and self.param == "mean")
                 and isinstance(self.point_est, dict)
                 and isinstance(self.variance, dict)
             ):
@@ -583,7 +623,7 @@ class TaylorEstimator(_SurveyEstimator):
         elif isinstance(self.variance, dict):
             for key in self.variance:
                 if (
-                    (self.parameter == "proportion" or as_factor and self.parameter == "mean")
+                    (self.param == "proportion" or as_factor and self.param == "mean")
                     and isinstance(self.point_est, dict)
                     and isinstance(self.variance, dict)
                 ):
@@ -644,6 +684,43 @@ class TaylorEstimator(_SurveyEstimator):
                     self.upper_ci[key] = self.point_est[key] + t_quantile * self.stderror[key]
                     self.coef_var[key] = math.sqrt(self.variance[key]) / self.point_est[key]
 
+    def _raise_singleton_error(self):
+        raise ValueError(f"Only one PSU in the following strata: {self.single_psu_strata}")
+
+    def _skip_singleton(self, skipped_strata: Array) -> Array:
+        skipped_str = np.isin(self.single_psu_strata, skipped_strata)
+        if skipped_str.sum() > 0:
+            return self.single_psu_strata[skipped_str]
+        else:
+            raise ValueError("{skipped_strata} does not contain singleton PSUs")
+
+    @staticmethod
+    def _certainty_singleton(
+        singletons: Array,
+        _stratum: Array,
+        _psu: Array,
+        _ssu: Array,
+    ) -> Array:
+        if _ssu is not None:
+            certainties = np.isin(_stratum, singletons)
+            _psu[certainties] = _ssu[certainties]
+        else:
+            for s in singletons:
+                cert_s = np.isin(_stratum, s)
+                nb_records = _psu[cert_s].shape[0]
+                _psu[cert_s] = np.linspace(1, nb_records, num=nb_records, dtype="int")
+
+        return _psu
+
+    @staticmethod
+    def _combine_strata(comb_strata: Array, _stratum: Array) -> Array:
+        if comb_strata is None:
+            raise ValueError("The parameter 'strata_comb' must be provided to combine strata")
+        else:
+            for s in comb_strata:
+                _stratum[_stratum == s] = comb_strata[s]
+            return _stratum
+
     def estimate(
         self,
         y: Array,
@@ -656,23 +733,26 @@ class TaylorEstimator(_SurveyEstimator):
         by: Optional[Series] = None,
         fpc: Union[dict[StringNumber, Number], Series, Number] = 1.0,
         deff: bool = False,
-        coef_variation: bool = False,
+        coef_var: bool = False,
+        single_psu: Union[SinglePSUEst, dict[StringNumber, SinglePSUEst]] = SinglePSUEst.error,
+        strata_comb: Optional[dict[Array, Array]] = None,
         as_factor: bool = False,
         remove_nan: bool = False,
     ) -> None:
 
-        if as_factor and self.parameter not in ("mean", "total"):
+        if as_factor and self.param not in ("mean", "total"):
             raise AssertionError("When as_factor is True, parameter must be mean or total!")
 
-        if self.parameter == "ratio" and x is None:
+        if self.param == "ratio" and x is None:
             raise AssertionError("x must be provided for ratio estimation.")
 
         y = numpy_array(y)
         y_temp = y.copy()
 
         x = numpy_array(x) if x is not None else None
-        psu = numpy_array(psu) if psu is not None else None
-        ssu = numpy_array(ssu) if ssu is not None else None
+        _stratum = numpy_array(stratum).copy() if stratum is not None else None
+        _psu = numpy_array(psu).copy() if psu is not None else None
+        _ssu = numpy_array(ssu).copy() if ssu is not None else None
 
         if samp_weight is None:
             weight_temp = np.ones(y_temp.shape[0])
@@ -684,17 +764,53 @@ class TaylorEstimator(_SurveyEstimator):
             weight_temp = np.asarray(samp_weight)
 
         if remove_nan:
-            if self.parameter == "ratio" and x is not None:
+            if self.param == "ratio" and x is not None:
                 excluded_units = np.isnan(y_temp) | np.isnan(x)
             else:
                 excluded_units = np.isnan(y_temp)
-            y_temp, weight_temp, x, stratum, psu, ssu, domain, by = remove_nans(
-                excluded_units, y_temp, weight_temp, x, stratum, psu, ssu, domain, by
+            y_temp, weight_temp, x, _stratum, _psu, _ssu, domain, by = remove_nans(
+                excluded_units, y_temp, weight_temp, x, _stratum, _psu, _ssu, domain, by
             )
 
-        if stratum is not None:
-            stratum = numpy_array(stratum)
-            self.strata = np.unique(stratum).tolist()
+        if _stratum is not None:
+            # _stratum = numpy_array(stratum).copy()
+            self.strata = np.unique(_stratum).tolist()
+            # TODO: we could improve efficiency by creating the pair [stratum,psu, ssu] ounce and
+            # use it in get_single_psu_strata and in the uncertainty calculation functions
+            self.single_psu_strata = get_single_psu_strata(_stratum, _psu)
+
+        skipped_strata = None
+        if self.single_psu_strata is not None:
+            if single_psu == SinglePSUEst.error:
+                self._raise_singleton_error()
+            if single_psu == SinglePSUEst.skip:
+                skipped_strata = self._skip_singleton(skipped_strata=self.single_psu_strata)
+            if single_psu == SinglePSUEst.certainty:
+                _psu = self._certainty_singleton(
+                    singletons=self.single_psu_strata, _stratum=_stratum, _psu=_psu, _ssu=_ssu
+                )
+            if single_psu == SinglePSUEst.combine:
+                _stratum = self._combine_strata(strata_comb, _stratum)
+            # TODO: more method for singleton psus to be implemented
+            if isinstance(single_psu, dict):
+                for s in single_psu:
+                    if single_psu[s] == SinglePSUEst.error:
+                        self._raise_singleton_error()
+                    if single_psu[s] == SinglePSUEst.skip:
+                        skipped_strata = self._skip_singleton(skipped_strata=numpy_array(s))
+                    if single_psu[s] == SinglePSUEst.certainty:
+                        _psu = self._certainty_singleton(
+                            singletons=numpy_array(s), _stratum=_stratum, _psu=_psu, _ssu=_ssu
+                        )
+                    if single_psu[s] == SinglePSUEst.combine:
+                        _stratum = self._combine_strata(strata_comb, _stratum)
+
+            skipped_strata = get_single_psu_strata(_stratum, _psu)
+            if skipped_strata is not None and single_psu in [
+                SinglePSUEst.certainty,
+                SinglePSUEst.combine,
+            ]:  # TODO: add the left our singletons when using the dict instead of SinglePSUEst
+                self._raise_singleton_error()
 
         if domain is not None:
             domain = numpy_array(domain)
@@ -707,7 +823,7 @@ class TaylorEstimator(_SurveyEstimator):
         if not isinstance(fpc, dict):
             self.fpc = fpc_as_dict(stratum, fpc)
         else:
-            if list(np.unique(stratum)) != list(fpc.keys()):
+            if list(np.unique(_stratum)) != list(fpc.keys()):
                 raise AssertionError("fpc dictionary keys must be the same as the strata!")
             else:
                 self.fpc = fpc
@@ -719,13 +835,14 @@ class TaylorEstimator(_SurveyEstimator):
                 y=y_temp,
                 samp_weight=weight_temp,
                 x=x,
-                stratum=stratum,
-                psu=psu,
-                ssu=ssu,
+                stratum=_stratum,
+                psu=_psu,
+                ssu=_ssu,
                 domain=domain,
                 fpc=self.fpc,
                 deff=deff,
-                coef_variation=coef_variation,
+                coef_var=coef_var,
+                skipped_strata=skipped_strata,
                 as_factor=as_factor,
                 remove_nan=remove_nan,
             )
@@ -735,28 +852,29 @@ class TaylorEstimator(_SurveyEstimator):
                 y_temp_b = y_temp[group_b]
                 weight_temp_b = weight_temp[group_b]
                 x_b = x[group_b] if x is not None else None
-                stratum_b = stratum[group_b] if stratum is not None else None
-                psu_b = psu[group_b] if psu is not None else None
-                ssu_b = ssu[group_b] if ssu is not None else None
+                _stratum_b = _stratum[group_b] if _stratum is not None else None
+                _psu_b = _psu[group_b] if _psu is not None else None
+                _ssu_b = _ssu[group_b] if _ssu is not None else None
                 domain_b = domain[group_b] if domain is not None else None
 
                 by_est = TaylorEstimator(
-                    parameter=self.parameter,
+                    param=self.param,
                     alpha=self.alpha,
-                    random_seed=self.random_seed,
+                    rand_seed=self.rand_seed,
                     ciprop_method=self.ciprop_method,
                 )
                 by_est._estimate(
                     y=y_temp_b,
                     samp_weight=weight_temp_b,
                     x=x_b,
-                    stratum=stratum_b,
-                    psu=psu_b,
-                    ssu=ssu_b,
+                    stratum=_stratum_b,
+                    psu=_psu_b,
+                    ssu=_ssu_b,
                     domain=domain_b,
                     fpc=self.fpc,
                     deff=deff,
-                    coef_variation=coef_variation,
+                    coef_var=coef_var,
+                    skipped_strata=skipped_strata,
                     as_factor=as_factor,
                     remove_nan=remove_nan,
                 )
@@ -766,14 +884,17 @@ class TaylorEstimator(_SurveyEstimator):
                 self.lower_ci[b] = by_est.lower_ci
                 self.upper_ci[b] = by_est.upper_ci
 
-    def to_dataframe(self, col_names: Optional(list) = None,) -> pd.DataFrame:
+    def to_dataframe(
+        self,
+        col_names: Optional(list) = None,
+    ) -> pd.DataFrame:
 
         if self.point_est is None:
             raise AssertionError("No estimates yet. Must first run estimate().")
         elif col_names is None:
-            if self.parameter == "proportion" or self.as_factor:
+            if self.param == "proportion" or self.as_factor:
                 col_names = [
-                    "_parameter",
+                    "_param",
                     "_domain",
                     "_level",
                     "_estimate",
@@ -785,7 +906,7 @@ class TaylorEstimator(_SurveyEstimator):
                 ]
             else:
                 col_names = [
-                    "_parameter",
+                    "_param",
                     "_domain",
                     "_estimate",
                     "_stderror",
@@ -828,6 +949,6 @@ class TaylorEstimator(_SurveyEstimator):
                 self.coef_var,
                 self.deff,
             )
-        est_df.iloc[:, 0] = self.parameter
+        est_df.iloc[:, 0] = self.param
 
         return est_df
