@@ -33,14 +33,14 @@ class ReplicateEstimator(_SurveyEstimator):
         | strata (list): list of the strata in the sample.
         | domains (list): list of the domains in the sample.
         | method (str): variance estimation method.
-        | parameter (str): the parameter of the population to estimate e.g. total.
-        | number_strata (int): number of strata.
-        | number_psus (int): number of primary sampling units (psus).
+        | param (str): the parameter of the population to estimate e.g. total.
+        | nb_strata (int): number of strata.
+        | nb_psus (int): number of primary sampling units (psus).
         | conservative (bool): indicate whether to produce conservative variance estimates.
-        | number_reps (int): number of replicate weights.
+        | nb_reps (int): number of replicate weights.
         | rep_coefs (array): coefficients associated to the replicate weights.
         | fay_coef (float): Fay coefficient for the the BRR-Fay algorithm.
-        | random_seed (int): random seed for reproducibility.
+        | rand_seed (int): random seed for reproducibility.
 
     Methods
         | estimate(): produces the point estimate of the parameter of interest with the associated
@@ -50,28 +50,28 @@ class ReplicateEstimator(_SurveyEstimator):
     def __init__(
         self,
         method: str,
-        parameter: str,
+        param: str,
         rep_weight_cls: Optional[Any] = None,
         fay_coef: Optional[Number] = None,
         alpha: float = 0.05,
-        random_seed: Optional[int] = None,
+        rand_seed: Optional[int] = None,
     ) -> None:
 
         if method.lower() not in ("bootstrap", "brr", "jackknife"):
             raise ValueError("method must be 'bootstrap', 'brr', or 'jackknife'")
 
-        super().__init__(parameter, alpha, random_seed)
+        super().__init__(param, alpha, rand_seed)
         self.method = method.lower()
         self.conservative = False
         self.degree_of_freedom: Optional[int] = None
-        self.number_reps: Optional[int] = None
+        self.nb_reps: Optional[int] = None
         self.rep_coefs: Optional[Union[np.ndarray, Number]] = None
         if method == "brr" and fay_coef is not None:
             self.fay_coef = fay_coef
         elif method == "brr":
             self.fay_coef = 0
         if rep_weight_cls is not None:
-            self.number_reps = rep_weight_cls.number_reps
+            self.nb_reps = rep_weight_cls.nb_reps
             self.rep_coefs = rep_weight_cls.rep_coefs
             self.degree_of_freedom = rep_weight_cls.degree_of_freedom
             self.fay_coef = rep_weight_cls.fay_coef if self.method == "brr" else None
@@ -79,13 +79,13 @@ class ReplicateEstimator(_SurveyEstimator):
     def _rep_point(
         self, y: np.ndarray, rep_weights: np.ndarray, x: Optional[np.ndarray]
     ) -> np.ndarray:
-        if self.parameter in ("proportion", "mean"):
+        if self.param in ("proportion", "mean"):
             return np.asarray(
                 np.sum(rep_weights * y[:, None], axis=0) / np.sum(rep_weights, axis=0)
             )
-        elif self.parameter == "total":
+        elif self.param == "total":
             return np.asarray(np.sum(rep_weights * y[:, None], axis=0))
-        elif self.parameter == "ratio" and x is not None:
+        elif self.param == "ratio" and x is not None:
             return np.asarray(
                 np.sum(rep_weights * y[:, None], axis=0) / np.sum(rep_weights * x[:, None], axis=0)
             )
@@ -114,16 +114,14 @@ class ReplicateEstimator(_SurveyEstimator):
                 self.rep_coefs = rep_coefs
             elif isinstance(rep_coefs, (int, float)):
                 self.rep_coefs = rep_coefs
-        elif self.number_reps is not None and self.method == "bootstrap":
-            self.rep_coefs = (1 / self.number_reps) * np.ones(self.number_reps)
-        elif self.number_reps is not None and self.method == "brr":
-            self.rep_coefs = (1 / (self.number_reps * pow(1 - self.fay_coef, 2))) * np.ones(
-                self.number_reps
-            )
-        elif self.number_reps is not None and self.method == "jackknife":
-            self.rep_coefs = ((self.number_reps - 1) / self.number_reps) * np.ones(
-                self.number_reps
+        elif self.nb_reps is not None and self.method == "bootstrap":
+            self.rep_coefs = (1 / self.nb_reps) * np.ones(self.nb_reps)
+        elif self.nb_reps is not None and self.method == "brr":
+            self.rep_coefs = (1 / (self.nb_reps * pow(1 - self.fay_coef, 2))) * np.ones(
+                self.nb_reps
             )
+        elif self.nb_reps is not None and self.method == "jackknife":
+            self.rep_coefs = ((self.nb_reps - 1) / self.nb_reps) * np.ones(self.nb_reps)
 
     def _variance(
         self,
@@ -174,7 +172,7 @@ class ReplicateEstimator(_SurveyEstimator):
     ) -> Any:
 
         if remove_nan:
-            if self.parameter == "ratio" and x is not None:
+            if self.param == "ratio" and x is not None:
                 excluded_units = np.isnan(y) | np.isnan(x)
             else:
                 excluded_units = np.isnan(y)
@@ -183,14 +181,14 @@ class ReplicateEstimator(_SurveyEstimator):
             )
             rep_weights = rep_weights[~excluded_units, :]
 
-        if self.parameter == "proportion":
+        if self.param == "proportion":
             y_dummies = pd.get_dummies(y)
             categories = y_dummies.columns
             y_dummies = y_dummies.values
 
         bias: Any
         if domain is None:
-            if self.parameter == "proportion" and categories is not None:
+            if self.param == "proportion" and categories is not None:
                 cat_dict = dict()
                 for k in range(categories.size):
                     cat_dict_k = dict(
@@ -208,11 +206,11 @@ class ReplicateEstimator(_SurveyEstimator):
             for d in np.unique(domain):
                 samp_weight_d = samp_weight * (domain == d)
                 rep_weights_d = rep_weights * (domain == d)[:, None]
-                if self.parameter == "ratio":
+                if self.param == "ratio":
                     x_d = x * (domain == d)
                 else:
                     x_d = x
-                if self.parameter == "proportion":
+                if self.param == "proportion":
                     y_dummies_d = y_dummies * (domain == d)[:, None]
                     cat_dict = dict()
                     for k in range(categories.size):
@@ -246,13 +244,13 @@ class ReplicateEstimator(_SurveyEstimator):
         remove_nan: bool = False,
     ) -> Any:
 
-        if self.parameter == "proportion":
+        if self.param == "proportion":
             y_dummies = pd.get_dummies(y)
             categories = y_dummies.columns
             y_dummies = y_dummies.values
 
         if domain is None:
-            if self.parameter == "proportion":
+            if self.param == "proportion":
                 estimate_k: dict
                 cat_dict = dict()
                 for k in range(categories.size):
@@ -280,11 +278,11 @@ class ReplicateEstimator(_SurveyEstimator):
             for d in np.unique(domain):
                 samp_weight_d = samp_weight * (domain == d)
                 rep_weights_d = rep_weights * (domain == d)[:, None]
-                if self.parameter == "ratio":
+                if self.param == "ratio":
                     x_d = x * (domain == d)
                 else:
                     x_d = x
-                if self.parameter == "proportion" and categories is not None:
+                if self.param == "proportion" and categories is not None:
                     y_dummies_d = np.asarray(y_dummies * (domain == d)[:, None])
                     cat_dict = dict()
                     for k in range(categories.size):
@@ -314,14 +312,14 @@ class ReplicateEstimator(_SurveyEstimator):
                         estimate_d,
                         conservative,
                     )
-            if self.parameter == "proportion":
+            if self.param == "proportion":
                 return variance_else1
             else:
                 return variance_else2
 
     def _get_confint(
         self,
-        parameter: str,
+        param: str,
         estimate: Any,
         variance: Any,
         quantile: float,
@@ -330,7 +328,7 @@ class ReplicateEstimator(_SurveyEstimator):
         lower_ci = {}
         upper_ci = {}
         if self.domains is None:
-            if parameter == "proportion":
+            if param == "proportion":
                 for level in variance:
                     point_est = estimate[level]
                     std_est = pow(variance[level], 0.5)
@@ -352,7 +350,7 @@ class ReplicateEstimator(_SurveyEstimator):
             lower_ci_else2 = {}
             upper_ci_else2 = {}
             for key in variance:
-                if parameter == "proportion":
+                if param == "proportion":
                     lower_ci_k = {}
                     upper_ci_k = {}
                     for level in variance[key]:
@@ -370,20 +368,20 @@ class ReplicateEstimator(_SurveyEstimator):
                     lower_ci_else2[key] = estimate[key] - quantile * pow(variance[key], 0.5)
                     upper_ci_else2[key] = estimate[key] + quantile * pow(variance[key], 0.5)
 
-            if self.parameter == "proportion":
+            if self.param == "proportion":
                 return lower_ci_else1, upper_ci_else1
             else:
                 return lower_ci_else2, upper_ci_else2
 
     def _get_coefvar(
         self,
-        parameter: str,
+        param: str,
         estimate: Any,  # Any := Union[dict[StringNumber, DictStrNum], DictStrNum, Number]
         variance: Any,
     ) -> Any:  # Any := Union[dict[StringNumber, DictStrNum], DictStrNum, Number]
 
         if self.domains is None:
-            if parameter == "proportion":
+            if param == "proportion":
                 coef_var = {}
                 for level in variance:
                     coef_var[level] = pow(variance[level], 0.5) / estimate[level]
@@ -392,7 +390,7 @@ class ReplicateEstimator(_SurveyEstimator):
         else:
             coef_var = {}
             for key in variance:
-                if parameter == "proportion":
+                if param == "proportion":
                     coef_var_k = {}
                     for level in variance[key]:
                         coef_var_k[level] = pow(variance[key][level], 0.5) / estimate[key][level]
@@ -433,7 +431,7 @@ class ReplicateEstimator(_SurveyEstimator):
             TypeRepEst: [description]
         """
 
-        if self.parameter == "ratio" and x is None:
+        if self.param == "ratio" and x is None:
             raise AssertionError("x must be provided for ratio estimation.")
 
         y = formats.numpy_array(y)
@@ -442,7 +440,7 @@ class ReplicateEstimator(_SurveyEstimator):
         x = formats.numpy_array(x) if x is not None else None
 
         if remove_nan:
-            if self.parameter == "ratio" and x is not None:
+            if self.param == "ratio" and x is not None:
                 excluded_units = np.isnan(y) | np.isnan(x)
             else:
                 excluded_units = np.isnan(y)
@@ -457,8 +455,8 @@ class ReplicateEstimator(_SurveyEstimator):
 
         self.conservative = conservative
 
-        if self.number_reps is None:
-            self.number_reps = rep_weights.shape[1]
+        if self.nb_reps is None:
+            self.nb_reps = rep_weights.shape[1]
 
         self._rep_coefs(rep_coefs)
 
@@ -480,26 +478,26 @@ class ReplicateEstimator(_SurveyEstimator):
         )
 
         if self.method == "brr" and self.degree_of_freedom is None:
-            self.degree_of_freedom = int(self.number_reps / 2)
+            self.degree_of_freedom = int(self.nb_reps / 2)
         elif self.degree_of_freedom is None:
-            self.degree_of_freedom = int(self.number_reps) - 1
+            self.degree_of_freedom = int(self.nb_reps) - 1
 
         t_quantile = student.ppf(1 - self.alpha / 2, df=self.degree_of_freedom)
 
         self.lower_ci, self.upper_ci = self._get_confint(
-            self.parameter, self.point_est, self.variance, t_quantile
+            self.param, self.point_est, self.variance, t_quantile
         )
-        self.coef_var = self._get_coefvar(self.parameter, self.point_est, self.variance)
+        self.coef_var = self._get_coefvar(self.param, self.point_est, self.variance)
 
         if self.domains is None:
-            if self.parameter == "proportion":
+            if self.param == "proportion":
                 for level in self.variance:
                     self.stderror[level] = pow(self.variance[level], 0.5)
             elif isinstance(self.variance, (int, float)):
                 self.stderror = pow(self.variance, 0.5)
         else:
             for key in self.variance:
-                if self.parameter == "proportion":
+                if self.param == "proportion":
                     stderror = {}
                     for level in self.variance[key]:
                         stderror[level] = pow(self.variance[key][level], 0.5)
